__author__ = 'DarkWeb'

# Here, we are importing the auxiliary functions to clean or convert data
from MarketPlaces.Utilities.utilities import *

# Here, we are importing BeautifulSoup to search through the HTML tree
from bs4 import BeautifulSoup
import re


# parses description pages, so takes html pages of description pages using soup object, and parses it for info it needs
# stores info it needs in different lists, these lists are returned after being organized
# @param: soup object looking at html page of description page
# return: 'row' that contains a variety of lists that each hold info on the description page
def atlas_description_parser(soup):
    # Fields to be parsed

    vendor = "-1"  # 0 *Vendor_Name y
    success = "-1"  # 1 Vendor_Successful_Transactions n
    rating_vendor = "-1"  # 2 Vendor_Rating y
    name = "-1"  # 3 *Product_Name y
    describe = "-1"  # 4 Product_Description y
    CVE = "-1"  # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures)
    MS = "-1"  # 6 Product_MS_Classification (Microsoft Security)
    category = "-1"  # 7 Product_Category y
    views = "-1"  # 8 Product_Number_Of_Views y
    reviews = "-1"  # 9 Product_Number_Of_Reviews n
    rating_item = "-1"  # 10 Product_Rating n
    addDate = "-1"  # 11 Product_AddedDate y
    BTC = "-1"  # 12 Product_BTC_SellingPrice y
    USD = "-1"  # 13 Product_USD_SellingPrice y
    EURO = "-1"  # 14 Product_EURO_SellingPrice n
    sold = "-1"  # 15 Product_QuantitySold n
    left = "-1"  # 16 Product_QuantityLeft n
    shipFrom = "-1"  # 17 Product_ShippedFrom n
    shipTo = "-1"  # 18 Product_ShippedTo n
    image = "-1"  # 19 Product_Image n
    vendor_image = "-1"  # 20 Vendor_Image n

    # Finding Vendor
    vendor = soup.find('div', {'class', 'flex items-center space-x-3'})
    vendor = vendor.find('div', {'class', 'flex items-center space-x-3'}).find('a').text
    vendor = cleanString(vendor).strip()

    # sold
    try:
        sell = soup.find('p', {'class': 'pl-3 text-sm font-semibold text-slate-700 dark:text-slate-400'}).text
        reg = r'product sold (.+)'
        sell = re.search(reg, sell)
        if sell:
            sell = sell.group(1)
        sell = sell.replace('product sold ', '').replace('time', '').replace('s', '')
        sell = cleanString(sell.strip())
    except:
        sell = '-1'
    sold = sell

    # successful transaction
    suc = soup.find('div', {'class': 'mt-4 grid w-full grid-cols-6 gap-4 rounded border p-3 shadow-sm dark:border-slate-700'}).find_all('div')
    suc = suc[2].find_all('p')
    suc = suc[1].text
    success = cleanString(suc.strip())


    # Finding Vendor Rating (bug in their system shows standard rating)
    rating_vendor = '-1'
    rating_vendor = cleanNumbers(rating_vendor).strip()

    # Finding Product Name
    name = soup.find('h2').text
    name = cleanString(name).strip()

    # Finding Product description
    try:
        describe = soup.find('div', {'class': 'prose mb-12 mt-5 break-words dark:prose-invert'}).text
        describe = cleanString(describe).strip()
    except:
        describe = '-1'

    # Finding category
    try:
        div_category = soup.find('ol', {'class': "leading-node flex items-center gap-1 text-sm font-medium text-gray-600 dark:text-gray-300"}).find_all('li', class_=lambda x: x is None)
        category = div_category[1].find('a').text
    except:
        category = '-1'
    category = cleanString(category).strip()

    # Product rating - check
    rating = '-1'
    rating_item = cleanString(rating).strip()

    # Finding BTC and USD/GOLD
    BTC = '-1'

    usd = soup.find('h1', {'class': 'text-2xl font-bold leading-none tracking-tight text-slate-800 dark:text-slate-300'}).text
    if '$' in usd:
        usd = usd.replace('$', '')
        usd = cleanString(usd).strip()
    USD = usd

    try:
        image = soup.find('div', {'class', 'w-full flex-1 flex-shrink-0'}).find('img').get('src').split('base64,')[-1]
    except:
        image = '-1'


    # Populating the final variable (this should be a list with all fields scraped)
    row = (vendor, rating_vendor, success, name, describe, CVE, MS, category, views, reviews, rating_item, addDate,
           BTC, USD, EURO, sold, left, shipFrom, shipTo, image, vendor_image)

    # Sending the results
    return row


# parses listing pages, so takes html pages of listing pages using soup object, and parses it for info it needs
# stores info it needs in different lists, these lists are returned after being organized
# @param: soup object looking at html page of listing page
# return: 'row' that contains a variety of lists that each hold info on the listing page
def atlas_listing_parser(soup):
    # Fields to be parsed
    nm = 0  # *Total_Products (Should be Integer)
    mktName = "Atlas"  # 0 *Marketplace_Name y
    vendor = []  # 1 *Vendor y
    rating_vendor = []  # 2 Vendor_Rating y
    success = []  # 3 Vendor_Successful_Transactions n
    name = []  # 4 *Product_Name y
    CVE = []  # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures) dont worry about this
    MS = []  # 6 Product_MS_Classification (Microsoft Security) dont worry about this
    category = []  # 7 Product_Category y
    describe = []  # 8 Product_Description n
    views = []  # 9 Product_Number_Of_Views y
    reviews = []  # 10 Product_Number_Of_Reviews n
    rating_item = []  # 11 Product_Rating n
    addDate = []  # 12 Product_AddDate y
    BTC = []  # 13 Product_BTC_SellingPrice y
    USD = []  # 14 Product_USD_SellingPrice y
    EURO = []  # 15 Product_EURO_SellingPrice n
    sold = []  # 16 Product_QuantitySold n
    qLeft = []  # 17 Product_QuantityLeft n
    shipFrom = []  # 18 Product_ShippedFrom n
    shipTo = []  # 19 Product_ShippedTo n
    image = []  # 20 Product_Image n
    image_vendor = []  # 21 Vendor_Image n
    href = []  # 22 Product_Links y

    listings = soup.find('div', {'class': 'grid h-fit grid-cols-1 gap-x-8 gap-y-10 lg:grid-cols-4 md:grid-cols-3'}).findAll('div', class_=lambda x: x is None)

    temp = soup.find('h1', {'class': 'text-4xl font-bold tracking-tight text-gray-900 dark:text-slate-200'}).text
    reg = r'results in (.+)'
    cat = re.search(reg, temp)
    if cat:
        cat = cat.group(1)
    cat = cat.replace('results in ', '')

    # Populating the Number of Products
    nm = len(listings)

    for listing in listings:
        # Finding vendor name
        vendor_name = listing.find('p', {'class': 'text-sm font-medium leading-none dark:text-slate-100'}).text
        vendor_name = vendor_name.strip()
        vendor.append(vendor_name)

        # Finding the vendor rating
        vendor_level = listing.find('p', {'class': "flex space-x-1 text-xs text-gray-700 dark:text-slate-500"}).text
        vendor_level = vendor_level.strip().split(' ')
        vendor_level = cleanNumbers(vendor_level[-1])
        rating_vendor.append(vendor_level)

        # Finding the product name
        product_name = listing.find('p', {'class':'my-1 line-clamp-2 text-sm text-slate-700 group-hover:underline dark:text-slate-300'}).text
        product_name = cleanString(product_name).strip()
        name.append(product_name)

        # Searching for CVE and MS categories
        cve = listing.findAll(text=re.compile('CVE-\d{4}-\d{4}'))
        if not cve:
            cveValue = "-1"
        else:
            cee = " "
            for idx in cve:
                cee += (idx)
                cee += "  "
                cee = cee.replace(',', ' ')
                cee = cee.replace('\n', '')
                cee = cee.replace('\t', '')
            cveValue = cee
        CVE.append(cveValue)

        ms = listing.findAll(text=re.compile('MS\d{2}-\d{3}'))
        if not ms:
            MSValue = "-1"
        else:
            me = " "
            for im in ms:
                me += (im)
                me += " "
                me = me.replace(',', ' ')
                me = me.replace('\n', '')
            MSValue = me
        MS.append(MSValue)

        # Finding the category
        category_text = cleanString(cat).strip()
        category.append(category_text)

        # Finding the hrefs
        description_link = listing.find('a', {'class': 'group relative block'})['href']
        href.append(description_link)

        # Finding the views - check
        views_text = '-1'
        views.append(views_text)

        # Finding the date added
        date = '-1'
        # date = datetime.strptime(date, "%d-%m-%Y")
        addDate.append(date)

        # EURO
        EURO.append('-1')

        # Finding the BTC and USD/GOLD
        btc_price = '-1'
        BTC.append(btc_price)

        try:
            money = listing.find('p', {'class': 'mt-2 text-lg font-bold tracking-wider text-slate-900 dark:text-slate-200'}).text
            if '$' in money:
                usd_price = money.strip().replace('$', '')
                usd_price = cleanString(usd_price).strip()
        except:
            usd_price = '-1'
        USD.append(usd_price)

        sold.append('-1')
        qLeft.append('-1')
        shipTo.append('-1')

        shipf = '-1'
        shipFrom.append(shipf)

        try:
            temp = listing.find('div', {'class', 'overflow-hidden rounded-lg'}).find('img').get('src').split('base64,')[-1]
        except:
            temp = '-1'
        image.append(temp)
        image_vendor.append('-1')

    # Populate the final variable (this should be a list with all fields scraped)
    return organizeProducts(mktName, nm, vendor, rating_vendor, success, name, CVE, MS, category, describe, views,
                            reviews, rating_item, addDate, BTC, USD, EURO, sold, qLeft, shipFrom, shipTo, href, image,
                            image_vendor)


# called by the crawler to get description links on a listing page
# @param: beautifulsoup object that is using the correct html page (listing page)
# return: list of description links from a listing page
def atlas_links_parser(soup):
    # Returning all links that should be visited by the Crawler

    href = []
    listings = soup.find('div', {'class': 'grid h-fit grid-cols-1 gap-x-8 gap-y-10 lg:grid-cols-4 md:grid-cols-3'}).findAll('div')

    for listing in listings:
        # Adding the url to the list of urls
        try:
            description_link = listing.find('a', {'class': 'group relative block'}).get('href')
            href.append(description_link)
        except:
            pass

    return href
