__author__ = 'DarkWeb'

# Here, we are importing the auxiliary functions to clean or convert data
from MarketPlaces.Utilities.utilities import *

# Here, we are importing BeautifulSoup to search through the HTML tree
from bs4 import BeautifulSoup


# parses description pages, so takes html pages of description pages using soup object, and parses it for info it needs
# stores info it needs in different lists, these lists are returned after being organized
# @param: soup object looking at html page of description page
# return: 'row' that contains a variety of lists that each hold info on the description page
def gofish_description_parser(soup):
    # Fields to be parsed

    vendor = "-1"                       # 0 *Vendor_Name
    success = "-1"                      # 1 Vendor_Successful_Transactions
    rating_vendor = "-1"                # 2 Vendor_Rating
    name = "-1"                         # 3 *Product_Name
    describe = "-1"                     # 4 Product_Description
    CVE = "-1"                          # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures)
    MS = "-1"                           # 6 Product_MS_Classification (Microsoft Security)
    category = "-1"                     # 7 Product_Category
    views = "-1"                        # 8 Product_Number_Of_Views
    reviews = "-1"                      # 9 Product_Number_Of_Reviews
    rating_item = "-1"                  # 10 Product_Rating
    addDate = "-1"                      # 11 Product_AddedDate
    BTC = "-1"                          # 12 Product_BTC_SellingPrice
    USD = "-1"                          # 13 Product_USD_SellingPrice
    EURO = "-1"                         # 14 Product_EURO_SellingPrice
    sold = "-1"                         # 15 Product_QuantitySold
    left = "-1"                         # 16 Product_QuantityLeft
    shipFrom = "-1"                     # 17 Product_ShippedFrom
    shipTo = "-1"                       # 18 Product_ShippedTo
    image = "-1"                        # 19 Product_Image
    vendor_image = "-1"                 # 20 Vendor_Image

    temp = soup.find('div', {'class': 'col-lg-5'})

    # find vendor name
    vendor = temp.find('a', {'class': 'text-decoration-none fw-bold'}).text.strip()
    if vendor is None:
        print('vendor')

    # find product name
    temp2 = soup.find('nav', {'aria-label': 'breadcrumb'}).findAll('li', {'class': 'breadcrumb-item'})
    name = soup.find('li', {'class': 'breadcrumb-item active text-truncate'}).text.strip()
    if name is None:
        print('name')

    describe = soup.find('div', {'class': 'p-3 mb-3 overflow-auto border border-2 bg-white rounded'}).text
    describe = cleanString(describe)
    if describe is None:
        print('describe')

    category = temp2[2].text
    if category is None:
        print('category')

    USD = soup.find('td', {'class': 'text-end text-nowrap'}).text
    if USD is None:
        print('USD')

    shipFrom = soup.find('span', {'class': 'lh-1 me-2 fs-4'}).text
    print(shipFrom)

    shipTo = soup.find('span', {'class': 'lh-1 me-2 fs-4'}).text
    if shipTo.isalnum():
        shipTo = shipTo
    else:
        shipTo = 'Worldwide'

    # Finding Product Image
    image = soup.find('figure', {"class": 'image-feature'}).find('img')
    if image is not None:
        image = image.get('src')
        image = image.split('base64,')[-1]
    else:
        print('img')
        image = "-1"


    # Populating the final variable (this should be a list with all fields scraped)
    row = (vendor, rating_vendor, success, name, describe, CVE, MS, category, views, reviews, rating_item, addDate,
           BTC, USD, EURO, sold, left, shipFrom, shipTo, image, vendor_image)

    # Sending the results
    return row


# parses listing pages, so takes html pages of listing pages using soup object, and parses it for info it needs
# stores info it needs in different lists, these lists are returned after being organized
# @param: soup object looking at html page of listing page
# return: 'row' that contains a variety of lists that each hold info on the listing page
def gofish_listing_parser(soup):

    # Fields to be parsed
    nm = 0  # *Total_Products (Should be Integer)
    mktName = "GoFish"                     # 0 *Marketplace_Name
    vendor = []                               # 1 *Vendor y
    rating_vendor = []                        # 2 Vendor_Rating
    success = []                              # 3 Vendor_Successful_Transactions
    name = []                                 # 4 *Product_Name y
    CVE = []                                  # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures) dont worry about this
    MS = []                                   # 6 Product_MS_Classification (Microsoft Security) dont worry about this
    category = []                             # 7 Product_Category y
    describe = []                             # 8 Product_Description
    views = []                                # 9 Product_Number_Of_Views
    reviews = []                              # 10 Product_Number_Of_Reviews
    rating_item = []                          # 11 Product_Rating
    addDate = []                              # 12 Product_AddDate
    BTC = []                                  # 13 Product_BTC_SellingPrice
    USD = []                                  # 14 Product_USD_SellingPrice y
    EURO = []                                 # 15 Product_EURO_SellingPrice
    sold = []                                 # 16 Product_QuantitySold
    qLeft = []                                # 17 Product_QuantityLeft
    shipFrom = []                             # 18 Product_ShippedFrom
    shipTo = []                               # 19 Product_ShippedTo
    image = []                                # 20 Product_Image
    image_vendor = []                         # 21 Vendor_Image
    href = []                                 # 22 Product_Links

    temp = soup.find('div', {"class": "col-9"})
    cat = temp.find('nav', {'aria-label': 'breadcrumb'}).find('li', {'class': 'breadcrumb-item active'}).text.strip()
    cat = cleanString(cat)

    listing = temp.find('tbody', {"class": 'border border-2 align-middle'}).findAll('tr')

    # Populating the Number of Products
    nm = len(listing)

    for a in listing:

        category.append(cat)

        # Adding the url to the list of urls
        link = a.find('a').get('href')
        link = cleanLink(link)
        href.append(link)

        # Finding the Product name
        product = a.find('a', {"class": 'text-decoration-none'}).text
        product = product.replace('\n', ' ')
        product = product.replace(",", "")
        product = product.replace("...", "")
        product = product.strip()
        name.append(product)

        # Finding Product Image
        product_image = a.find('img')
        product_image = product_image.get('src')
        product_image = product_image.split('base64,')[-1]
        image.append(product_image)

        # Finding the Vendor
        vendor_name = a.find('a', {"class": 'text-decoration-none fw-bold'}).text
        vendor_name = vendor_name.replace(",", "")
        vendor_name = vendor_name.strip()
        vendor.append(vendor_name)

        # image vendor
        image_vendor.append("-1")

        # USD
        usd = a.find('div', {'class': 'text-nowrap'}). find('span', {'class': 'fw-bold text-nowrap'}).text.strip()
        USD.append(usd)

        temp = a.findAll('span', {'class': 'fs-4 lh-1'})
        shipF = temp[0].text
        shipFrom.append(shipF)

        shipT = temp[1].text

        if shipT.isalnum():
            shipTo.append(shipT)
        else:
            shipTo.append('Worldwide')



        rating_vendor.append('-1')
        success.append('-1')
        CVE.append('-1')
        MS.append('-1')
        describe.append('-1')
        views.append('-1')
        reviews.append('-1')
        rating_item.append('-1')
        addDate.append('-1')
        BTC.append('-1')
        EURO.append('-1')
        sold.append('-1')
        qLeft.append('-1')

    # Populate the final variable (this should be a list with all fields scraped)
    return organizeProducts(mktName, nm, vendor, rating_vendor, success, name, CVE, MS, category, describe, views,
                            reviews, rating_item, addDate, BTC, USD, EURO, sold, qLeft, shipFrom, shipTo, href, image, image_vendor)


# called by the crawler to get description links on a listing page
# @param: beautifulsoup object that is using the correct html page (listing page)
# return: list of description links from a listing page
def gofish_links_parser(soup):
    # Returning all links that should be visited by the Crawler

    href = []
    listing = soup.find('div', {"class": "col-9"}).find('tbody', {'class': 'border border-2 align-middle'}).findAll('tr')

    for a in listing:
        bae = a.findAll('a', href=True)

        # Adding the url to the list of urls
        link = bae[0].get('href')
        href.append(link)

    return href