__author__ = 'DarkWeb'

# Here, we are importing the auxiliary functions to clean or convert data
from MarketPlaces.Utilities.utilities import *

# Here, we are importing BeautifulSoup to search through the HTML tree
from bs4 import BeautifulSoup


# parses description pages, so takes html pages of description pages using soup object, and parses it for info it needs
# stores info it needs in different lists, these lists are returned after being organized
# @param: soup object looking at html page of description page
# return: 'row' that contains a variety of lists that each hold info on the description page
def nemesis_description_parser(soup):
    # Fields to be parsed

    vendor = "-1"                       # 0 *Vendor_Name
    success = "-1"                      # 1 Vendor_Successful_Transactions
    rating_vendor = "-1"                # 2 Vendor_Rating
    name = "-1"                         # 3 *Product_Name
    describe = "-1"                     # 4 Product_Description
    CVE = "-1"                          # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures)
    MS = "-1"                           # 6 Product_MS_Classification (Microsoft Security)
    category = "-1"                     # 7 Product_Category
    views = "-1"                        # 8 Product_Number_Of_Views
    reviews = "-1"                      # 9 Product_Number_Of_Reviews
    rating_item = "-1"                  # 10 Product_Rating
    addDate = "-1"                      # 11 Product_AddedDate
    BTC = "-1"                          # 12 Product_BTC_SellingPrice
    USD = "-1"                          # 13 Product_USD_SellingPrice
    EURO = "-1"                         # 14 Product_EURO_SellingPrice
    sold = "-1"                         # 15 Product_QuantitySold
    left = "-1"                         # 16 Product_QuantityLeft
    shipFrom = "-1"                     # 17 Product_ShippedFrom
    shipTo = "-1"                       # 18 Product_ShippedTo
    image = "-1"                        # 19 Product_Image
    vendor_image = "-1"                 # 20 Vendor_Image

    # find vendor name
    vendor = soup.find('div', {'class': 'd-flex align-items-center mt-n1'}).find('a').text
    vendor = cleanString(vendor).strip()

    # find product name
    name = soup.find('div', {'class': 'd-flex w-100 mt-4'}).find('a').text
    name = name.replace(r'[^a-zA-Z0-9]', '').strip()

    # find product description
    describe = soup.find('div', {'class': 'card card-bordered rounded-1 my-6'})\
        .find('div', {'class': 'fs-5 text-gray-800'}).text
    if describe is not None:
        describe = cleanString(describe).strip()
    else:
        describe = soup.find('div', {'class': 'card card-bordered rounded-1 my-6'}) \
            .find('div', {'class': 'fs-5 text-gray-800 mt-5'}).text
        if describe is not None:
            describe = cleanString(describe).strip()
        else:
            describe = "-1"

    # find product category
    temp = soup.find('div', {'class': 'fs-7 py-1'}).findAll('a')
    category = temp[1].text
    if category is not None:
        category = category.strip()
    else:
        category='-1'
        print('inside category')


    # all in class
    temp = soup.find('div', {'class': 'ms-4'}).findAll('div')

    # number of reviews
    reviews = temp[1].text
    if reviews is not None:
        reviews = reviews.replace("Reviews: ","").strip()
    else:
        reviews='-1'
        print('in rev')

    # item rating out of 5
    rating_item = temp[0].text
    if rating_item is None:
        rating_item = rating_item.replace("Rating: ","").replace(" out of 5","").strip()
    else:
        rating_item='-1'



    # amount of item sold
    success = temp[2].text
    if success is not None:
        success = success.replace('Sales: ', '').strip()
    else:
        success='-1'
        print('success')

    # amount
    temp = soup.find('div', {'class': 'text-gray-800 fs-1 fw-bolder mt-6'}).text

    if 'USD' in temp:
        USD = temp.replace('USD', '').strip()
    elif 'EUR' in temp:
        EURO = temp.replace('EUR', '').strip()


    # Finding Vendor Image
    vendor_image = soup.find('div', {"class": 'col'}).find('img')
    if vendor_image is not None:
        vendor_image = vendor_image.get('src')
        vendor_image = vendor_image.split('base64,')[-1]
    else:
        vendor_image = "-1"

    # Finding Product Image
    image = soup.find('div', {"class": 'container p-0'}).find('img')
    if image is not None:
        image = image.get('src')
        image = image.split('base64,')[-1]
    else:
        image = "-1"

    # Searching for CVE and MS categories
    cve = soup.findAll(text=re.compile('CVE-\d{4}-\d{4}'))
    if cve:
        CVE = " "
        for idx in cve:
            CVE += (idx)
            CVE += "  "
            CVE = CVE.replace(',', ' ')
            CVE = CVE.replace('\n', '')
    ms = soup.findAll(text=re.compile('MS\d{2}-\d{3}'))
    if ms:
        MS = " "
        for im in ms:
            MS += (im)
            MS += " "
            MS = MS.replace(',', ' ')
            MS = MS.replace('\n', '')

    # Populating the final variable (this should be a list with all fields scraped)
    row = (vendor, rating_vendor, success, name, describe, CVE, MS, category, views, reviews, rating_item, addDate,
           BTC, USD, EURO, sold, left, shipFrom, shipTo, image, vendor_image)

    # Sending the results
    return row


# parses listing pages, so takes html pages of listing pages using soup object, and parses it for info it needs
# stores info it needs in different lists, these lists are returned after being organized
# @param: soup object looking at html page of listing page
# return: 'row' that contains a variety of lists that each hold info on the listing page
def nemesis_listing_parser(soup):

    # Fields to be parsed
    nm = 0  # *Total_Products (Should be Integer)
    mktName = "NemesisMarket"                     # 0 *Marketplace_Name
    vendor = []                               # 1 *Vendor y
    rating_vendor = []                        # 2 Vendor_Rating
    success = []                              # 3 Vendor_Successful_Transactions
    name = []                                 # 4 *Product_Name y
    CVE = []                                  # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures) dont worry about this
    MS = []                                   # 6 Product_MS_Classification (Microsoft Security) dont worry about this
    category = []                             # 7 Product_Category y
    describe = []                             # 8 Product_Description
    views = []                                # 9 Product_Number_Of_Views
    reviews = []                              # 10 Product_Number_Of_Reviews
    rating_item = []                          # 11 Product_Rating
    addDate = []                              # 12 Product_AddDate
    BTC = []                                  # 13 Product_BTC_SellingPrice
    USD = []                                  # 14 Product_USD_SellingPrice y
    EURO = []                                 # 15 Product_EURO_SellingPrice
    sold = []                                 # 16 Product_QuantitySold
    qLeft = []                                # 17 Product_QuantityLeft
    shipFrom = []                             # 18 Product_ShippedFrom
    shipTo = []                               # 19 Product_ShippedTo
    image = []                                # 20 Product_Image
    image_vendor = []                         # 21 Vendor_Image
    href = []                                 # 22 Product_Links

    cat = soup.find('span', {"class": "text-gray-700 fw-bold fs-2"}).text
    cat = cleanString(cat).strip()

    listing = soup.find('div', {"class": 'row g-5 g-xl-5'}).findAll('div', {"class": 'col-sm-6 col-md-4 col-lg-3 col-xxl-2'})

    # Populating the Number of Products
    nm = len(listing)

    for a in listing:

        category.append(cat)

        # Adding the url to the list of urls
        link = a.find('a').get('href')
        link = cleanLink(link)
        href.append(link)

        # Finding the Product name
        product = a.find('div', {"class": 'pt-3 pb-4 px-4'}).find('a').text.strip()
        # product = product.replace(r'[^a-zA-Z0-9]', '').strip()
        name.append(product)

        # Finding Product Image
        product_image = a.find('img')
        if product_image is not None:
            product_image = product_image.get('src')
            product_image = product_image.split('base64,')[-1]
            image.append(product_image)
        else:
            image.append("-1")

        # Finding Prices
        price = a.find('div', {"class": "fs-4 text-gray-800 fw-bolder mt-3"}).text
        price = price.strip()
        if 'USD' in price:
            price = price.replace('USD', '')
            price = price.strip()
            USD.append(price)
            EURO.append("-1")
        elif 'EUR' in price:
            price = price.replace('EUR', '')
            price = price.strip()
            EURO.append(price)
            USD.append("-1")
        else:
            USD.append("-1")
            EURO.append('-1')

        # Finding Item Rating
        temp = a.find('div', {"class": "d-flex flex-column"}).findAll('span', {"class": "badge badge-sm badge-light-dark text-gray-800 rounded-1"})
        rating = temp[0].text.strip()
        if rating is not None:
            rating_item.append(rating)
        else:
            rating_item.append('-1')
            print('rating')

        # find number of reviews
        rev = temp[1].text.strip()
        if rev is not None:
            reviews.append(rev)
        else:
            reviews.append('-1')
            print('reviews')

        # sales
        sales = temp[2].text.strip()
        if sales is not None:
            success.append(sales)
        else:
            success.append('-1')
            print('success')


        # Finding the Vendor
        vendor_name = a.find('div', {"class": "d-flex flex-column"}).find('a').text
        vendor_name = vendor_name.replace(",", "")
        vendor_name = vendor_name.strip()
        vendor.append(vendor_name)

        # vendor image
        vendor_image = a.find('div', {"class": 'd-flex align-items-center'}).find('a').find('img')
        if vendor_image is not None:
            vendor_image = vendor_image.get('src')
            vendor_image = vendor_image.split('base64,')[-1]
            image_vendor.append(vendor_image)
        else:
            image_vendor.append("-1")

        # item description
        description = a.find('div', {"class": 'fs-5 text-gray-800 mt-2'}).text

        if description is not None:
            description = description.replace('\n', ' ')
            describe.append(description)
        else:
            describe.append('-1')
            print('describe')

        # Searching for CVE and MS categories
        cve = a.findAll(text=re.compile('CVE-\d{4}-\d{4}'))
        if not cve:
            cveValue = "-1"
        else:
            cee = " "
            for idx in cve:
                cee += (idx)
                cee += "  "
                cee = cee.replace(',', ' ')
                cee = cee.replace('\n', '')
            cveValue = cee
        CVE.append(cveValue)

        ms = a.findAll(text=re.compile('MS\d{2}-\d{3}'))
        if not ms:
            MSValue = "-1"
        else:
            me = " "
            for im in ms:
                me += (im)
                me += " "
                me = me.replace(',', ' ')
                me = me.replace('\n', '')
            MSValue = me
        MS.append(MSValue)

        val="-1"
        sold.append(val)
        views.append(val)
        addDate.append(val)
        BTC.append(val)
        qLeft.append(val)
        shipFrom.append(val)
        shipTo.append(val)
        rating_vendor.append(val)

    # Populate the final variable (this should be a list with all fields scraped)
    return organizeProducts(mktName, nm, vendor, rating_vendor, success, name, CVE, MS, category, describe, views,
                            reviews, rating_item, addDate, BTC, USD, EURO, sold, qLeft, shipFrom, shipTo, href, image, image_vendor)


# called by the crawler to get description links on a listing page
# @param: beautifulsoup object that is using the correct html page (listing page)
# return: list of description links from a listing page
def nemesis_links_parser(soup):
    # Returning all links that should be visited by the Crawler

    href = []
    listing = soup.findAll('div', {"class": "col-sm-6 col-md-4 col-lg-3 col-xxl-2"})

    for a in listing:
        bae = a.findAll('a', href=True)

        # Adding the url to the list of urls
        link = bae[0].get('href')
        href.append(link)

    return href
